﻿using System;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;

namespace CountMeIn.Interactivity
{
    public class ExecuteCommandAction : TriggerAction<FrameworkElement>
    {
        public static readonly DependencyProperty CommandNameProperty =
             DependencyProperty.Register("CommandName", typeof(string), typeof(ExecuteCommandAction), null);

        public static readonly DependencyProperty CommandParameterProperty =
            DependencyProperty.Register("CommandParameter", typeof(object), typeof(ExecuteCommandAction), null);

        protected override void Invoke(object parameter)
        {
            if(AssociatedObject == null)
                return;

            ICommand command = null;

            var dataContext = AssociatedObject.DataContext;

            foreach(var info in dataContext.GetType().GetProperties(BindingFlags.Public | BindingFlags.Instance))
            {
                if(IsCommandProperty(info) && String.Equals(info.Name, CommandName, StringComparison.Ordinal))
                {
                    command = (ICommand)info.GetValue(dataContext, null);
                    break;
                }
            }

            if((command != null) && command.CanExecute(CommandParameter))
            {
                command.Execute(CommandParameter);
            }
        }

        private static bool IsCommandProperty(PropertyInfo property)
        {
            return typeof(ICommand).IsAssignableFrom(property.PropertyType);
        }

        public string CommandName
        {
            get
            {
                return (string)GetValue(CommandNameProperty);
            }
            set
            {
                SetValue(CommandNameProperty, value);
            }
        }

        public object CommandParameter
        {
            get
            {
                return GetValue(CommandParameterProperty);
            }
            set
            {
                SetValue(CommandParameterProperty, value);
            }
        }

    }
}
